#import "DirectoryBrowserItem.h"
#import "Utility.h"


@implementation DirectoryBrowserItem


@synthesize name		= mName;
@synthesize path		= mPath;
@synthesize image		= mImage;
@synthesize numImages	= mNumImages;


-(id)initWithPath:(NSString *)path
{
	self = [super init];
	if (self != nil)
	{
		mPath		= [path copy];
		mChildren	= nil;
		mImage		= nil;
		mName		= [[mPath lastPathComponent] retain];
		mNumImages	= 0;
	}
	return self;
}

-(void)dealloc
{
	[mPath release];
	[mName release];
	[mChildren removeAllObjects];
	[mChildren release];
	[mImage release];
	[super dealloc];
}

-(NSMutableArray *)children
{
	// first time children is accessed, initliaze its content
	if (mChildren == nil)
	{
		mChildren = [[NSMutableArray alloc] init];
		NSFileManager * fileManager = [[NSFileManager alloc] init];
		NSArray * content = [fileManager contentsOfDirectoryAtPath:mPath error:nil];
		BOOL isDirectory;
		for (NSString * file in content)
		{
			// ignore hidden directories
			if ([file characterAtIndex:0] == '.')
				continue;
			
			NSString * fullPath = [mPath stringByAppendingPathComponent:file];
			if ([fileManager fileExistsAtPath:fullPath isDirectory:&isDirectory] == YES)
			{
				if (isDirectory == YES)
				{
					DirectoryBrowserItem * item = [[DirectoryBrowserItem alloc] initWithPath:fullPath];
					[mChildren addObject:item];
					[item release];
				}
				else if ([Utility isImage:fullPath] == YES)
					++mNumImages;
			}
		}
		[fileManager release];
	}
	return mChildren;
}

@end
