#import "ImageBrowser.h"
#import "../ImageView/ImageView.h"


@implementation ImageBrowser


@synthesize imageView	= mImageView;
@synthesize window		= mWindow;


//! override this to initialize a few member
-(void)awakeFromNib
{
	mSlideshowTimer = nil;
}

//! This is used the NSTimer callback, used by the slideshow
-(void)slideShow:(NSTimer *)timer
{
	[self changeImage:1 loop:[[timer userInfo] boolValue]];
}

//! Utility method used to change image
-(void)changeImage:(int)direction loop:(BOOL)loop
{
	// compute the new index
	int index = [[self selectionIndexes] firstIndex] + direction;

	// check the bounds
	int numImages = [[self delegate] numberOfItemsInImageBrowser:self];
	if (index < 0)
		index = (loop == YES ? index + numImages : 0);
	if (index >= numImages)
		index = (loop == YES ? index - numImages : numImages - 1);

	// set the new selected index
	[self setSelectionIndexes:[NSIndexSet indexSetWithIndex:index] byExtendingSelection:NO];
}

//! we use the ImageBrowser as the main event handler. All keyboard event are handled here,
//! and the other parts of the UI eventually redirect their events to this.
-(void)keyDown:(NSEvent *)theEvent
{
	NSString * characters = [theEvent characters];
	NSUInteger modifier = [theEvent modifierFlags];
	unichar event = [characters characterAtIndex:0];
	BOOL bFullscreen = [mImageView isInFullScreenMode];
	
	switch (event)
	{
		case 27:
			if (mSlideshowTimer != nil)
			{
				[mSlideshowTimer invalidate];
				mSlideshowTimer = nil;
			}

			// Escap : toggles fullscreen only if we're in fullscreen
			if (bFullscreen == YES)
			{
				[mImageView toggleFullscreen];
				[self scrollIndexToVisible:[[self selectionIndexes] firstIndex]];
			}				
			else
				[super keyDown:theEvent];
			break;
			
		case 13:
			if (mSlideshowTimer != nil)
			{
				[mSlideshowTimer invalidate];
				mSlideshowTimer = nil;
			}

			// Enter : toggles fullscreen
			[mImageView toggleFullscreen];
			break;
			
		// up & left : advance backward
		case NSUpArrowFunctionKey:
		case NSLeftArrowFunctionKey:
		{
			if (bFullscreen == NO)
			{
				[super keyDown:theEvent];
				break;
			}

			// check modifiers
			if (modifier & NSShiftKeyMask)			[self changeImage:-5 loop:NO];
			else if (modifier & NSControlKeyMask)	[self changeImage:-10 loop:NO];
			else if (modifier & NSCommandKeyMask)	[self changeImage:-20 loop:NO];
			else									[self changeImage:-1 loop:NO];

			break;
		}

		// down & right : advance forward
		case NSDownArrowFunctionKey:
		case NSRightArrowFunctionKey:
		{
			if (bFullscreen == NO)
			{
				[super keyDown:theEvent];
				break;
			}
			
			// check modifiers
			if (modifier & NSShiftKeyMask)			[self changeImage:5 loop:NO];
			else if (modifier & NSControlKeyMask)	[self changeImage:10 loop:NO];
			else if (modifier & NSCommandKeyMask)	[self changeImage:20 loop:NO];
			else									[self changeImage:1 loop:NO];

			break;
		}

		// space : stops the slideshow
		case ' ':
			if (mSlideshowTimer == nil)
				break;

		case 'p':
		{
			// check if a slideshow is already going
			if (mSlideshowTimer != nil)
			{
				[mSlideshowTimer invalidate];
				mSlideshowTimer = nil;
				break;
			}
			else
			{
				// if nothing's selected, select the first image
				if ([self selectionIndexes] == nil || [[self selectionIndexes] count] == 0)
					[self setSelectionIndexes:[NSIndexSet indexSetWithIndex:0] byExtendingSelection:NO];
				// if still nothing's selected, it means we don't have any image in the image browser, so
				// simply break (launching a slideshow on an empty folder has no meaning :)
				if ([self selectionIndexes] == nil || [[self selectionIndexes] count] == 0)
					break;
				
				// set fullscreen if it's not already the case
				if (bFullscreen == false)
					[mImageView toggleFullscreen];
				
				// get the slideshow user preferences
				NSUserDefaultsController * defaults = [NSUserDefaultsController sharedUserDefaultsController];
				BOOL loop = [[[defaults values] valueForKey:@"slideshowLoop"] boolValue];
				float interval = [[[defaults values] valueForKey:@"slideshowInterval"] floatValue];
			
				// launch the slideshow. I use a simple NSTimer with scheduledTimerWithTimeInterval.
				// This method creates the timer and automatically fire it after "interval" secondes.
				mSlideshowTimer = [NSTimer scheduledTimerWithTimeInterval:interval
														  target:self
														selector:@selector(slideShow:)
														userInfo:[NSString stringWithFormat:@"%s", (loop ? "Y" : "N")]
														 repeats:YES];
				[mSlideshowTimer setFireDate:[NSDate dateWithTimeIntervalSinceNow:interval]];
			}

			break;
		}

		default:
			[super keyDown:theEvent];
	}		
}

-(BOOL)becomeFirstResponder
{
	// override this method so that when the image browser becomes the first
	// responder, we select the first image of the browser
	[self setSelectionIndexes:[NSIndexSet indexSetWithIndex:0] byExtendingSelection:NO];
	return YES;
}

-(BOOL)resignFirstResponder
{
	// when asked to let go of the first responder status, check the fullscreen mode :
	// if it's on, don't let go ! If we let go, the directory browser view will get
	// the messages instead of us !
	if ([mImageView isInFullScreenMode] == YES)
		return NO;
	return YES;
}

@end
